/*
 * Name: main.c
 * Version: 1.0
 * Author: Andreas Mehler http://www.apm-motionpictures.de
 * Date: 09/11/12
 * License: Creative Commons Attribution-Share Alike 3.0
 *
 * This program implements a remote control for synchronized
 * start/stop of the Canon DSLR 60D (and compatible) and
 * the Zoom H4n handy recorder.
 *
 * Built and tested with an ATmega168 @ 9.8304MHz
 *
 * More information is available at
 * http://www.apm-motionpictures.de/projects/
 *
 */
#include <avr/io.h>
#include <util/delay.h>
 
/* timings for camera remote control */
#define HPERIOD 0.01524
#define NPULSES 16

/* USART baud rate - 2400 for H4n */
#define	BAUD 2400

/* H4n LED bitmasks */
#define LED_REC 0x01
#define LED_CLIP_MIC 0x02
#define LED_CLIP_CH1 0x04
#define LED_CLIP_CH2 0x08

/* global status flag */
volatile uint8_t recording = 0;

/* because of avr-libc 1.4 compatibilty */ 
void long_delay(uint16_t ms)
{
    for(; ms>0; ms--) _delay_ms(1);
}

/* initialize IO */
void io_init(void)
{
	PORTC |= (1 << PC0); // init pin with 'high'
    DDRC |= (1 << PC0); // (out) IR LED

	PORTD |= (1 << PD6) | (1 << PD7); //init pins with 'high'
	DDRD |= (1 << PD6) | (1 << PD7); // (out) 6-Record 7-clipping

	DDRB &= ~(1 << PB0); // (in) trigger button
}

/* initialize USART */
void uart_init(void)
{
	/*  Set baud rate generator based on F_CPU */
	UBRR0H = (unsigned char)(F_CPU/(16UL*BAUD)-1)>>8;
	UBRR0L = (unsigned char)(F_CPU/(16UL*BAUD)-1);
	/* Enable USART0 transmitter and receiver */
	UCSR0B = (1<<RXEN0) | (1<<TXEN0);
}

/* send char */
void uart_putchar(unsigned char data)
{
	while(!(UCSR0A & (1 << UDRE0))) // wait until USART is ready for tx
		;       // do nothing
	UDR0 = data;    // send char
}

/* toggle camera rec/stop */
void toggle_camera(void)
{
	uint8_t i;

	for(i=0; i<=NPULSES; i++)
	{
		PORTC &= ~(1 << PC0); //IR on
		_delay_ms(HPERIOD);
		PORTC |= (1 << PC0); //IR off
		_delay_ms(HPERIOD);
	}

	_delay_ms(5.36);

	for(i=0; i<=NPULSES; i++)
	{
		PORTC &= ~(1 << PC0); //IR on
		_delay_ms(HPERIOD);
		PORTC |= (1 << PC0); //IR off
		_delay_ms(HPERIOD);
	}
}

/* send record command */
void h4n_record(void)
{
	unsigned char command[4];
	command[0] = 0x81;
	command[1] = 0x00;
	command[2] = 0x80;
	command[3] = 0x00;
	uart_putchar(command[0]); //press
	uart_putchar(command[1]);
	_delay_ms(10);
	uart_putchar(command[2]); //release
	uart_putchar(command[3]);
}

/* send stop command */
void h4n_stop(void)
{
	unsigned char command[4];
	command[0] = 0x84;
	command[1] = 0x00;
	command[2] = 0x80;
	command[3] = 0x00;
	uart_putchar(command[0]); //press
	uart_putchar(command[1]);
	_delay_ms(10);
	uart_putchar(command[2]); //release
	uart_putchar(command[3]);
}

/* toggle recorder rec/stop */
void toggle_recorder(void)
{
	if(recording) { //stop
		h4n_stop();
		recording = 0;
	} else {  //start
		h4n_record();
		_delay_ms(20);
		_delay_ms(20);
		h4n_record();
		recording = 1;
	}
}

int main(void)
{
	unsigned char buffer;
	uint8_t bufval, ready;
	ready = 0;

	io_init();
	uart_init();

	/* registration sequence */
	while(!ready)
	{
		buffer = 0x00;
		uart_putchar(buffer);
		/* try to get char from UART */
		if((UCSR0A & (1 << RXC0))) {
			buffer = UDR0;
			if(buffer == 0x80) {
				buffer = 0xA1;
				uart_putchar(buffer);
				ready = 1;
			}
		}
	}

	/* main loop */
	while(1)
    {
		/* handle USART */
		if((UCSR0A & (1 << RXC0)))
		{
			buffer = UDR0;
			bufval = (uint8_t)buffer;

			/* handle clipping led */
			if(bufval & (LED_CLIP_CH1 | LED_CLIP_CH2 | LED_CLIP_MIC))
				PORTD &= ~(1 << PD7);
			else
				PORTD |= (1 << PD7);

			/* handle record led */
			if(bufval & LED_REC)
				PORTD &= ~(1 << PD6);
			else
				PORTD |= (1 << PD6);
		}

		/* handle trigger button */
		if(PINB & (1 << PB0))
		{
			toggle_camera();
			toggle_recorder();
			while(PINB & (1 << PB0))
			;// wait for button release
			long_delay(400); //debounce + time for cam and H4n to start/stop
		}
    }
 
    return 0;
}
